//
// rpwRand.cpp
//

#include "rpwRand.h"

#include<iostream>
#include <cmath>
#include <cstdlib>
#include <climits>

using namespace std;

#ifdef HAVE_RANDOM
#define RANDOM (random())
#define SRANDOM(x) (srandom(x))
#define RANDOM_MAX INT_MAX
#else
#define RANDOM (rand())
#define SRANDOM(x) (srand(x))
#define RANDOM_MAX RAND_MAX
#endif


//--------------------------------------------
//  Generic wrapper routines for picking
// random numbers.
//--------------------------------------------

void initRand(int seed)
{
  SRANDOM(seed);
} // init_rand


// Select a random number between 0 and n-1
int iRand(int n)
{
  return ( RANDOM % n );
} // irand
 

// Select a random float between 0.0 and 1.0
float fRand(void)
{
  return ( (float) RANDOM / (float) RANDOM_MAX );
} // frand


// Select a random double between 0.0 and 1.0
double dRand(void)
{
  return ( (double) RANDOM / (double) RANDOM_MAX );
} // drand



//--------------------------------------------
//  Routines for drawing random permutations
//--------------------------------------------
//  Basic idea is to take a list 0..n-1, pick
// a random position, swap it with the 0th
// element, increment the position and pick
// a new position in the remaining 1..n-1 
// positions.  Repeat until at (n-1)st pos.
//
//  The result is a permutation selected
// uniformly at random from the n! space
// of permutations of sequences of size
// n.  This is done is \Theta(n) time.
//--------------------------------------------


// Initialize the arrow by filling it with 
// consecutive numbers from 0 to n-1
void initArray(int *A, int n)
{
  for (int i =0; i<n; i++)
    A[i] = i;
} // initArray


// Swap random position with current 
// position, then increment current.
int swapNInc (int *A, int n, int pos)
{
  int newPos, tmp;

    // draw a new position in pos..n-1
    newPos = pos + iRand(n-pos);
    
    // swap old pos and new pos
    tmp       = A[pos];
    A[pos]    = A[newPos];
    A[newPos] = tmp;

  return(pos+1);  
}


void permRand (int *A, int n)
{
  initArray(A,n);

  int i = 0;
  while (i < (n-1))   // the last pos has 0 degrees of freedom
    i = swapNInc(A,n,i);
}




